"""CLI tool for showing audit logs."""

import requests
import typer

from app.console import get_console
from app.constants import API_KEY_NAME, ApiPaths, API_REQUEST_TIMEOUT_SEC
from app.utils.checks import require_api_endpoint_and_key
from app.utils.datetime_utils import format_utc_to_local
from rich.table import Table

audit_app = typer.Typer()


@audit_app.command(name='display', help='Display audit logs')
@require_api_endpoint_and_key()
def display_audit_logs(
    ctx: typer.Context,
):
    """Display audit logs with pagination."""

    api_endpoint = ctx.obj['api_endpoint']
    api_key = ctx.obj['api_key']

    audit_url = f'{api_endpoint}{ApiPaths.ACCESS_AUDIT_LOGS}'
    headers = {API_KEY_NAME: api_key}

    page = 1
    size = 50  # Default page size, can be adjusted as needed
    items = []
    while True:
        response = requests.get(
            audit_url,
            headers=headers,
            params={'page': page, 'size': size},
            timeout=API_REQUEST_TIMEOUT_SEC,
        )
        response.raise_for_status()

        audit_data = response.json()
        items.extend(audit_data.get('items', []))

        if not audit_data.get('pages') or page >= audit_data['pages']:
            break

        page += 1

    console = get_console()

    if not items:
        console.print('No audit logs found.')
        return

    table = Table(title='Audit Logs')
    table.add_column('Username', justify='center')
    table.add_column('Method', justify='left')
    table.add_column('Path', justify='left')
    table.add_column('Client IP', justify='right')
    table.add_column('Timestamp', justify='right')

    for item in items:
        created_at = item.get('created_at_utc')
        timestamp = format_utc_to_local(created_at) if created_at else 'N/A'
        table.add_row(
            item.get('username', 'N/A'),
            item.get('method', 'N/A'),
            item.get('path', 'N/A'),
            item.get('client_ip', 'N/A'),
            timestamp,
        )

    console.print(table)
